@@ -11,7 +11,8 @@ meta.alpha=.5, norm.method=0, norm.blur=2, norm.minG=10
                         dat$clsEvents, dat$M, dat$S, 
                         bias=meta.bias, sub.thres=0, I.iter=meta.iter, B=50, 
                         tol=tol, EM.method=20, alpha=meta.alpha,
-                        norm.method=norm.method, norm.blur=norm.blur, 
+                        ## norm-stuff obsolet here
+                        norm.method=norm.method, norm.blur=norm.blur,
                         norm.minG=norm.minG)
 
     dat.norm <- dat
@@ -30,16 +31,18 @@ meta.alpha=.5, norm.method=0, norm.blur=2, norm.minG=10
                                 rep(0.0,npar(res))), 1, mean)
     attr(res, "limits") <- attr(exp[[1]], "limits")[,dat.subset]
 
-    meta <- list("dat.scatter"=NULL, "res.scatter"=NULL,  
-                    "dat.clusters"=dat, "res.clusters"=res)
-    meta$gating <- list("clusters"=seq_len(res@K), "childs"=c(),
-                    "desc"="all", "partition"=TRUE)
-
+    #meta <- list("dat.scatter"=NULL, "res.scatter"=NULL,
+    #                "dat.clusters"=dat, "res.clusters"=res)
+    #meta$gating <- list("clusters"=seq_len(res@K), "childs"=c(),
+    #                "desc"="all", "partition"=TRUE)
+    #
+    #
+    #meta$gating$pscales <- Default_Scales(attr(res, "trans.a"),
+    #                                    attr(res, "limits"))
+    #
+    #class(meta) <- "immunoMeta"
     
-    meta$gating$pscales <- Default_Scales(attr(res, "trans.a"),
-                                        attr(res, "limits"))
-    
-    class(meta) <- "immunoMeta"
+    meta <- immunoMeta(res, dat)
     meta
 }
 ## meta.process
@@ -150,7 +153,7 @@ bias=0.25, alpha=0.5, min.class=0
                     K=L, P=P, w=obj$w[seq_len(L)], mu=mu, sigma=sigma, 
                     z=z, label=obj$label, 
                     logLike=obj$logLike, BIC=BIC, ICL=ICL,
-                    state=obj$history[seq_len(L)])
+                    history=paste(obj$history[seq_len(L)]) )
     
     result
     
@@ -163,8 +166,10 @@ bias=0.25, alpha=0.5, min.class=0
 ###
 meta.Clustering <- function(
 P, N, K, W, M, S, label=NULL, I.iter=10, B=500, tol=1e-5, 
-bias=0.25, sub.thres=bias, alpha=0.5, EM.method=20,
-norm.method=0, norm.blur=2, norm.minG=10, verbose=FALSE
+bias=0.25, sub.thres=bias, alpha=0.5, 
+EM.method=20, HC.samples=2000,
+norm.method=0, norm.blur=2, norm.minG=10, ## norm-stuff obsolet here
+verbose=FALSE
 ) {
 
     totK <- sum(K)
@@ -189,6 +194,7 @@ norm.method=0, norm.blur=2, norm.minG=10, verbose=FALSE
     ### sub-clustering is performed with EM.method=20
     if( EM.method == 23 )
         subEM.method <- 20
+    
     for( i in seq_len(I.iter) ) {
         if( norm.method > 0 ) {
             if( G >= norm.minG) {
@@ -204,30 +210,45 @@ norm.method=0, norm.blur=2, norm.minG=10, verbose=FALSE
             }
         }
         
-        #if( G < 10 )
-        #label <- meta.SubClustering(P, totK, W, tM, tS, label, tol=tol,
-        #                            bias=bias*0.5, alpha=alpha,
-        #                            EM.method=subEM.method)
-        #else
-        #label <- meta.SubClustering(P, totK, W, tM, tS, label, tol=tol,
-        #                            bias=bias, alpha=alpha,
-        #                            EM.method=subEM.method)
+        
+        btm <- ptm <- proc.time()
+        
         if( res@K < 10 )
         sub_bias <- bias*0.5
         else
-        sub_bias <- bias
+        sub_bias <- sub.thres
         label <- meta.SubClustering(res, P, totK, W, tM, tS,
                                     tol=tol,
                                     bias=sub_bias, thres=sub.thres, alpha=alpha,
-                                    EM.method=subEM.method,
+                                    EM.method=subEM.method, 
+                                    HC.samples=HC.samples,
                                     verbose=verbose)
-        
+        if( verbose ) {
+            message("sub-clustering (", i, "/", I.iter, ") => ", 
+            length(unique(res@label)), "=>", length(unique(label)),
+            "clusters takes ", 
+            paste( round((etm<-proc.time()) - ptm), collapse=",") )
+            ptm <- etm
+        }
+        else {
         message("Fit Model ", i, " of ", I.iter, " with ",
-                length(unique(label)), " clusters")
-        res <- meta.ME(P, N, K, W, tM, tS, label, B=B, tol=tol, 
+            length(unique(label)), " clusters")
+        }
+        res <- meta.ME(P, N, K, W, tM, tS, label, B=B, tol=tol,
                         bias=bias, alpha=alpha, method=EM.method)
+        
+        if( verbose ) {
+            message("meta-clustering (", i, "/", I.iter, ") => ", 
+            res@K, "/", length(unique(label(res))),
+            " clusters takes ", 
+            paste( round((etm<-proc.time()) - ptm), collapse=",") )
+            ptm <- etm
+        }
+        else {
         message("=> results in  ", res@K, " clusters")
-        label <- res@label 
+        }
+        res@state <- label
+        label <- res@label
         
         if( res@K == G ) break
         G <- res@K
@@ -236,25 +257,39 @@ norm.method=0, norm.blur=2, norm.minG=10, verbose=FALSE
     res
 }
 
+## const delta cost (without information clustering assigment)
+.icl_delta_costs <- function(N,P,K, L) {
+## delta for L sub cluster if one cluster
+    res <- (L-1)*(P*(P+1)/2 + P)*log(N)*0.5
+    res <- res - lgamma((K+L-1)/2) + lgamma(K/2)
+    res <- res + (L-1) * lgamma(1/2)
+    res <- res + lgamma(N+(K+L-1)/2) - lgamma(N+K/2)
+    res
+}
 
-#meta.SubClustering <- function(
-#P, N, W, M, S, label, tol=1e-5, bias=0.25, alpha=1.0, EM.method=20
 meta.SubClustering <- function(
-x, P, N, W, M, S, tol=1e-5, bias=0.25, thres=bias, alpha=1.0, EM.method=20,
+x, P, N, W, M, S, tol=1e-5, bias=0.25, thres=bias, alpha=1.0, 
+EM.method=20, HC.samples=2000,
 verbose=FALSE
 ) {
     
     label <- x@label
     K <- x@K
-    #K <- max(label)
+    sum_T <- sum(W)
+    if( EM.method==200 )
+        sum_T <- length(W)
     
+    ## maximal sub clustering count for one cluster
     J <- 8
-    cutoff <- 0
     
-    ## 2022.010.10: find a balance to introduce more clusters
+    ## 2022.10.10: find a balance to introduce more clusters
     ##thres <- bias
-    icl_thres <- (P*(P+1)/2 + P)*log(sum(W))*0.5*thres
-    #message("ICL thres ",  format(icl_thres, digits=2) )
+    icl_thres <- (P*(P+1)/2 + P)*log(sum_T)*0.5*thres
+    if( verbose ) {
+        message("ICL thres ",  format(icl_thres, digits=2), " (", thres, ")" )
+    }
+    
+    cutoff <- 0
     
     res_l <- vector("list", K)
     icl_l <- rep(0, K)
@@ -263,28 +298,60 @@ verbose=FALSE
     for( k in seq_len(K) ) {
         
         inc <- which(label==k)
+        
+        if(verbose) {
+            message("cluster ", k, " test with ", length(inc), " obs and ", 
+            sum(W[inc]), " evts" )
+        }
         if( length(inc) > 1 ) {
             res <- meta.TestSubCluster(x,
                     P, length(inc), W[inc], M[inc,], S[inc,],
                     J=min(length(inc),J), tol=tol,
-                    bias=bias, alpha=alpha, EM.method=EM.method )
+                    bias=bias, alpha=alpha, 
+                    EM.method=EM.method, HC.samples=HC.samples )
+                    
+            if( is.null(res) && verbose ) {
+                message("cluster ", k, " null TestSubCluster")
+            }
         }
         else {
             res <- NULL
         }
+        
+        
         res_l[[k]] <- res
 
         if( !is.null(res) && length(res) > 1 ) {
             
             icl <- rep(0, length(res))
-            for( l in seq_len(length(res)) )
-            icl[l] <- res[[l]]@ICL/res[[l]]@K
+            for( l in seq_len(length(res)) ) {
+                ## 2024.05.08:
+                ##icl[l] <- res[[l]]@ICL/res[[l]]@K
+                icLike <- res[[l]]@logLike[4] ## delta icLike
+                iclSum <- res[[l]]@logLike[3] ## delta iclSum  .icl_delta_costs
+                clCost <- .icl_delta_costs(sum_T, P, K, res[[l]]@K)
+                
+                #icCost <- iclSum - .icl_delta_costs(sumW, P, K, res[[l]]@K)
+                icl[l] <- icLike + iclSum - bias*clCost
+                
+                if( verbose ) {
+                    message(k,"/", l,": ", round(icl[l]), "(L=", res[[l]]@K,
+                        " icl=", round(icLike), " sum=", round(iclSum),
+                        " costs=", round(clCost), ")")
+                }
+            }
+            
+            icl[1] <- cutoff
             
             icl_l[k] <- max(icl)
             l <- which.max(icl)
             tst_l[k] <- l
         }
         else {
+            if( verbose ) {
+                message("cluster ", k, " tested ", length(res))
+            }
+            
             icl_l[k] <- cutoff
             tst_l[k] <- 1
         }
@@ -308,20 +375,21 @@ verbose=FALSE
         }
         
         if( res[[l]]@K > 1 && verbose ) {
-            message(J, "/", sK, ": cluster ", k, " has ", res[[l]]@K, 
-                " sub-clusters at ", l, ", ICL=", format(icl, digits=2), "<>",
-                format(icl_thres, digits=2))
+            message(J, "/", sK, ": cluster ", k, " (obs=", sum(label==k), 
+                ") has ", res[[l]]@K, " sub-clusters at ", l,
+                ", ICL=", format(icl, digits=2) )
         }
         
         icl_l[k] <- cutoff
         
         res <- res[[l]]
         
-        if( icl <= cutoff )
-        break
+        if( icl <= cutoff ) {
+            break
+        }
         
         ## 2022.10.17: respect cluster costs
-        if( (res@K>1) && (icl>icl_thres) ) {
+        if( (res@K>1) ) { ## && (icl>icl_thres) ) {
             ins[[k]] <- res
         }
         
@@ -338,11 +406,21 @@ verbose=FALSE
 ## keep label k, the rest get's a new label
 ## should have the consequence, that the cluster label of the first experiment
 ## is not changed if they all distinct??? (required for EM.method=23)
-        inc <- which(label==k)
+        k_obs <- which(label==k)
         lnc <- (ins[[k]]@label==ins[[k]]@label[1])
-        knc <- inc[lnc]
-        label[label==k] <- ins[[k]]@label + max(label)
-        label[knc] <- k
+        f_obs <- k_obs[lnc]
+        if( verbose ) {
+            message("cls ", k, " (", ins[[k]]@K, " subs on ", length(k_obs), 
+            " obs): +max=", max(label))
+            
+        }
+        label[k_obs] <- ins[[k]]@label + max(label)
+        label[f_obs] <- k
+        
+        if( verbose ) {
+            message( "stable sub-k=", ins[[k]]@label[1], "(for ", sum(lnc),
+            " obs) => max=", max(label))
+        }
     }  #for cluster k
     
     label
@@ -351,10 +429,9 @@ verbose=FALSE
 
 meta.TestSubCluster<-
 function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
-        EM.method=2, HC.samples=2000)
+        EM.method=20, HC.samples=2000)
 {
-#message("meta.TestSubCluster: ", "N=", N, " J=", J, 
-#           " dim=", paste(sep="", dim(M), collapes=","))
+
     if( J > N ) {
         warning("\t???", J, "sub >", N, "clusters")
         return (NULL)
@@ -367,7 +444,7 @@ function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
     if( is.null(parameters) ) {
         parameters <- paste(sep="", "P", seq_len(P))
     }
-    result <- vector("list", J)
+    result <- vector("list")
     
     label <- rep(1, N)
 
@@ -379,6 +456,7 @@ function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
                 as.double(bias), as.double(alpha), as.integer(1))
 
     if( obj$L < 1 ) {
+        
         return(NULL)
     }
 
@@ -400,8 +478,14 @@ function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
 # output BIC & ICL
     BIC <- obj$logLike[1]
     ICL <- 0
+    ## 3 = icLike + iclSum
     logLike <- obj$logLike[3]
-        
+    
+    ## 4 = icLike
+    icLike <- obj$logLike[4]
+    ## 3-4 = iclSum
+    iclSum <- obj$logLike[3] - obj$logLike[4]
+    obj$logLike[2] <- obj$logLike[3] <- obj$logLike[4] <- 0
         
 # output
     result[[1]] <- new("immunoClust", parameters=parameters,
@@ -428,8 +512,9 @@ function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
                                 }) > -100)
         
         
+            
         if( J > length(samples.set) ) {
-            return (NULL)
+            return (result)
         }
         
         if( length(samples.set) > HC.samples ) {
@@ -443,11 +528,15 @@ function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
                                         M[samples.set,], S[samples.set,])
         }
         
-        for (k in 2:J) {
-                
-            label <- rep(0, N)
-                
-            label[samples.set] <- .clust.hclass(hcPairs, k)
+        
+        #for (k in 2:J) {
+        #for( j  in 2:J )
+        j <- 2
+        {
+            
+            k <- J+2-j
+                        label <- rep(0, N)
+            label[samples.set] <- .meta.hclass(hcPairs, k)
                 
             obj <- .Call("immunoC_metaME",
                         as.integer(N), as.integer(P), as.integer(k),
@@ -457,7 +546,9 @@ function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
                         as.double(bias), as.double(alpha), as.integer(1))
 
             L <- obj$L
-
+            
+            #if( L == 1 ) break
+            
 # output obj$s to sigma
             sigma <- array(0, c(L, P, P))
             s <- matrix(obj$s, k, P * P, byrow=TRUE)
@@ -473,12 +564,22 @@ function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
             }
 # output BIC & ICL
             BIC <- obj$logLike[1]
-                ## 2022.10.10: .icl_delta ???
-            #ICL <- obj$logLike[3] - logLike
-            ICL <- obj$logLike[3] - logLike - .icl_delta(sumW, P, K, L)*bias
-                
+            ## 2022.10.10: .icl_delta ???
+            ## 2024.05.08:
+            ## delta icLike with icCost
+            ICL <- obj$logLike[2] - logLike
+            ##ICL <- obj$logLike[3] - logLike - 
+            ##    .icl_delta_cor(sumW, P, K, L)*bias
+            
+            ## delta logLike
+            ## 3: icLike + iclSum => delta iclSum
+            ## 4: icLike
+            obj$logLike[3] <- obj$logLike[3] - obj$logLike[4] - iclSum
+            ## 4: icLike => delta icLike
+            obj$logLike[4] <- obj$logLike[4] - icLike
+            
 # outp
-            result[[k]] <- new("immunoClust", parameters=parameters,
+            result[[j]] <- new("immunoClust", parameters=parameters,
                             K=L, N=N, P=P, w=obj$w, mu=mu, sigma=sigma,
                             label = obj$label,
                             logLike=obj$logLike, BIC=BIC, ICL=ICL)
@@ -498,7 +599,7 @@ function(x, P, N, W, M, S, J=8, B=500, tol=1e-5, bias=0.5, alpha=1.0,
 ###
 # meta.hclust
 ###
-meta.hclust <- function(P, N, W, M, S) 
+meta.hclust <- function(P, N, W, M, S)
 {
     partition <- seq_len(N)
     attr(partition, "unique") <- N
