@@ -12,10 +12,11 @@
 #include "util.h"
 
 #include <gsl/gsl_cblas.h>
-#include <gsl/gsl_math.h>
+//#include <gsl/gsl_math.h>
+#include <gsl/gsl_machine.h> // FLT_MAX
 #include <algorithm>
 #include <gsl/gsl_errno.h>
-#include <gsl/gsl_sf_gamma.h>
+
 
 using std::max;
 using std::min;
@@ -974,21 +975,27 @@ em_mvt::m_step_diag_k(int k)
 	// co-variance -> precision
 
 	for(p=0; p<P; ++p) {
-		if( (*(s+p*P+p) /= z_sum) <= EPSMIN ) {
-			// 2014.04.29: 				
+        /*
+        if( *(s+p*P+p) == 0 && z_sum > 3 ) {
+            status = 1;
+            break;
+        }
+        */
+        if( (*(s+p*P+p) /= z_sum) <= EPSMIN ) {
+			// 2014.04.29:
 			//dbg::printf("%d: singularity in co-variance parameter %d (%g => %g, z-sum %.4lf)", k, p, (*(s+p*P+p))*z_sum, TRC[p]*z_sum, z_sum);
 			//				status = 1;
 			//				break;
 			//				*(s+p*P+p) = one/TRC;
 			*(s+p*P+p) = TRC[p];
 		}
-        /*
+        // 2024.05.23: doch ein unterschied??
         // ueberflussig: kann gar nicht sein nach definition von TRC
 		if( *(s+p*P+p) <= EPSMIN ) {
 			status = 1;
 			break;
 		}
-        */
+        
 		double srt = sqrt(*(s+p*P+p));
 		*(s+p*P+p) = 1.0/srt; 
 	}
