@@ -20,18 +20,22 @@ class mvn_dendro
 	double*	W;	// K
 	double*	M;	// K x P
 	double* S;	// K x P x P
+    
+    const double    zero;
+    
 	double*	D;	// K*(K-1)/2
 	int*	CLS;
 	
-	double*	tmpS;
-	double* tmpPxP;
-	double* tmpP;
+	double*	tmpS;       // PxP
+	double* tmpPxP;     // PxP
+	double* tmpP;       // P
 	
 public:
 	mvn_dendro(int k, int p, double* w, double* m, double* s);
 	~mvn_dendro();
 	
 	int hellinger(int* li, int* lj, double* crit);
+    int hellinger_fast(int* li, int* lj, double* crit, double alpha=1); // ist nicht faster
 	int mahalanobis(int* li, int* lj, double* crit);
 	int hellinger_d(int* li, int* lj, double* crit);
 	
@@ -39,12 +43,20 @@ public:
 	int mahalanobis_w(int* li, int* lj, double* crit);
 	
 private:
+
+    void    init_D();
+    void    init_D_diag(double alpha);
+    void    update_D(int oi, int oj);
+    void    update_D_diag(int oi, int oj, double alpha);
+    
 	void	swap_nodes(int j, int l);
 	void	join_nodes(int i, int j);
 	double	joined_ij(int i, int j, double* M_ij, double* S_ij) const;
 	void	joined_invS(int i, int j);
 	
 	double	logdet_invS(const double* S, int& status);
+    double  logdet_S(const double* S, int& status);
+    
 	void	inv_sumS(const double* S_i, const double* S_j);
 	int		weighted_linkage(int* li, int* lj, double* crit);
 
